﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Input;
using TimeScheduler.Common;
using TimeScheduler.Domain;
using TimeScheduler.Model;

namespace TimeScheduler.ViewModel
{
  class MainViewModel : NotifyableObject
  {
    #region Konstruktion
    /// <summary>Standard-Konstruktor</summary>
    public MainViewModel() : this(new Domain.Impl.FileDomain()) { Refresh(); }

    /// <summary>Konstruktor</summary>
    /// <param name="provider"></param>
    public MainViewModel(IDomain provider)
    {
      Provider = provider;

      TimeItems = new ObservableCollection<ITimeItem>();
      TimeItems.CollectionChanged += TimeItems_CollectionChanged;

      CreateCommands();
    }

    /// <summary>Datenbeschaffer</summary>
    protected IDomain Provider { get; private set; }
    #endregion

    #region Eigenschaften
    private DateTime currentDate_ = DateTime.Now.Date;
    public DateTime CurrentDate
    {
      get { return currentDate_; }
      set
      {
        if (SetField(ref currentDate_, value))
          RefreshForDate(currentDate_);
      }
    }

    private Dictionary<int, string> costUnits_;
    public Dictionary<int, string> CostUnits { get { return costUnits_; } private set { SetField(ref costUnits_, value); } }

    private Dictionary<TimeItemType, string> itemTypes_;
    public Dictionary<TimeItemType, string> ItemTypes { get { return itemTypes_; } private set { SetField(ref itemTypes_, value); } }

    private ObservableCollection<ITimeItem> timeItems_;
    /// <summary>Beinhaltet die Elemente des aktuell ausgewählten Tages</summary>
    public ObservableCollection<ITimeItem> TimeItems { get { return timeItems_; } private set { SetField(ref timeItems_, value); } }

    private ITimeItem selectedTimeItem_;
    /// <summary>Das aktuell selektierte Element, von dem weitere Details angezeigt werden sollen</summary>
    public ITimeItem SelectedTimeItem { get { return selectedTimeItem_; } set { SetField(ref selectedTimeItem_, value); } }
    #endregion

    #region Kommandos
    private void CreateCommands()
    {
      RefreshCommand = new RelayCommand(Refresh);
      AddNewCommand = new RelayCommand(AddNew);
      DeleteCommand = new RelayCommand(Delete, CanDelete);
    }

    public ICommand RefreshCommand { get; private set; }
    private void Refresh()
    {
      // Standardwerte belegen
      CostUnits = Provider.GetCostUnits();
      var it = new Dictionary<TimeItemType, string>();
      foreach (TimeItemType tit in Enum.GetValues(typeof(TimeItemType)))
        it[tit] = Enum.GetName(typeof(TimeItemType), tit);
      ItemTypes = it;

      // Andere Daten ermitteln
      RefreshForDate(CurrentDate);
    }
    private void RefreshForDate(DateTime date)
    {
      foreach (var item in TimeItems)
        item.PropertyChanged -= Item_PropertyChanged;

      // Laufende Datein belegen
      var items = Provider.GetItems(date);

      // Event abhängen, damit keine Änderungen an den Provider geschickt werden
      TimeItems.CollectionChanged -= TimeItems_CollectionChanged;

      try
      {
        // Event der Objekte entfernen
        foreach (var item in TimeItems)
          item.PropertyChanged -= Item_PropertyChanged;

        // Elemente umlagern
        TimeItems.Clear();
        foreach (var item in items)
        {
          item.PropertyChanged += Item_PropertyChanged;
          TimeItems.Add(item);
        }
      }
      finally
      {
        // Event wieder anhängen
        TimeItems.CollectionChanged += TimeItems_CollectionChanged;
      }

      SelectedTimeItem = TimeItems.FirstOrDefault();
    }

    public ICommand AddNewCommand { get; private set; }
    private void AddNew() { TimeItems.Add(Provider.NewItem()); }

    public ICommand DeleteCommand { get; private set; }
    private void Delete() { TimeItems.Remove(SelectedTimeItem); }
    public bool CanDelete() { return SelectedTimeItem != null; }
    #endregion

    #region Hilfsfunktionen
    private void TimeItems_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
    {
      if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
      {
        foreach (ITimeItem item in e.NewItems)
          item.Key = Provider.ElementAdded(item);
      }
      else if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
      {
        foreach (ITimeItem item in e.OldItems)
          Provider.ElementRemoved(item);
      }
    }

    private void Item_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
    {
      Provider.ElementChanged((ITimeItem)sender);
    }
    #endregion
  }
}
